==> driza-0.5.2.tar.gz: driza-0.5.2/Driza/datos.py <==
#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Datos: Todas las clases relacionadas con los datos, y sus correspondientes metodos

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import variables
import types
from qt import QString

class Registro(list):
	"""Almacena un registro, permitiendo el acceso tanto por el indice, como por el nombre de variable. 
	Esta clase garantiza que la lista de valores se ajusta a los tipos de la lista de variables
	Hereda de list"""

	def __init__(self,variables):
		""" 
		Clase constructora
		variables es el array de variables al que esta asociado el registro
		Añade los valores por defecto para todos los atributos
		"""
		#VARIABLES PRIVADAS
		self.__variables=variables

		list.__init__(self)
		for x in self.__variables.arr_defaults():
			self.append(x)	
	
	#FUNCIONES PUBLICAS
	def __getitem__(self,indice):
		""" Redefinicion del [] de la lista. 
		Si nos pasan una ristra, comprobamos si se corresponde a alguna variable
		Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
		Si nos pasan un entero, devolvemos el valor de esa posicion
		"""
		miindice=self.__variables.obtenerindice(indice)
		return list.__getitem__(self,miindice)

	def __setitem__(self,indice,valor):
		""" 
		Redefinicion de []= de la lista.  Almacena en el valor convertido a formato interno por la variable asociada
		"""
		miindice=self.__variables.obtenerindice(indice)
		if not isinstance(valor,str):
			valor=repr(valor)
		list.__setitem__(self,miindice,self.__variables[miindice].nuevoitem(valor))

	def append(self,valor):
		""" 
		Redefinicion del append de lista. Añade al registro el valor pasado en el formato interno de la variable asociada 
		"""
		#TODO: Este append es problemático, puesto que depende de que la variable ya exista y ademas sea la última. Pendiente de rediseñar
		intermedio=self.__variables[self.__len__()].nuevoitem(valor)
		list.append(self,intermedio)
		
	def set(self,lista):
		""" Esta funcion establece los valores del registro, teniendo como argumento una lista o derivado """
		i=0
		for valor in lista:
			self.__setitem__(i,valor)
			i+=1


class ListaVar(list): 
	"""ListaVar hereda de list. Su utilidad es almacenar un conjunto de variables, permitiendo recuperarlas no solo por el indice, sino por el nombre."""

	def __init__(self):
		list.__init__(self)

	def __getcopy__(self):
		resultado=ListaVar()
		for i in self:
			resultado.append(i)
		return resultado

	#FUNCIONES PUBLICAS

	def arr_defaults(self):
		"""Devuelve una lista con las ristras de todos los valores por defecto"""
		return [x.valorpordefecto for x in self]

	def obtenerindice(self,parametro):
		"""
 		Permite obtener el indice de una variable a partir de su nombre ,del objeto, o de un entero.
		Devuelve un entero.
		"""
		if isinstance(parametro,QString):
			parametro=parametro.latin1()

		if parametro.__class__==types.StringType:
			for iter in self:  
				if iter.nombre == parametro:
					return self.index(iter)
			return False
		elif parametro.__class__==types.IntType:
			return parametro
		else: #El parametro lo tomamos como una referencia
			if parametro in self:
				return self.index(parametro)
			return False


class ContenedorRegVar:
	"""Esta clase almacena los datos del programa. Tiene dos miembros, una lista de registros y una lista de variables"""
	def __init__(self,registros=None,variables=None):
		"""Inicialización del contenedor principal"""
		if not registros: 
			registros=[]
		self.registros=registros #Un array para datos
		if not variables: 
			variables=ListaVar()
		self.variables=variables #Un array de variables 

	def __deepcopy__(self,memo):
		"""Reimplementación de __deepcopy__, permite una copia del objeto"""
		nuevalista=self.variables.__getcopy__()
		nuevalistaregistros=list()
		for registro in self.registros:
			nuevoregistro=Registro(nuevalista)
			for x in range(len(registro)):
				nuevoregistro[x]=nuevalista[x].nuevoitem(repr(registro[x]))
			nuevalistaregistros.append(nuevoregistro)
		resultado= ContenedorRegVar(nuevalistaregistros,nuevalista)
		memo[id(self)]=resultado
		return resultado




class ListaUndoRedo(list):
	"""ListaUndoRedo hereda de lista. Es similar a una cola lifo"""
	def __init__(self,tama):
		self.tama=tama
	def encolar(self,objeto,index=0):
		"""Si pasan un indice es que quieren encolar desde alguna posición"""
		if index>=self.tama:
			pass #error
		else:
			if self.__len__()==self.tama:
				self.__delitem__(-1)
			self.__delslice__(0,index)
			self.insert(0,objeto)

class PorteroDatos:
	"""Clase de portera. Gestiona una lista con todas las copias de los datos, determinando cual es la vigente y permitiendo el undo y el redo"""
	def __init__(self):
		self.__index=0 #Variable que lleva el indice
		self.lista=ListaUndoRedo(5)
		self.lista.encolar(ContenedorRegVar())

	def undo(self):
		"""Realiza un undo, desplazando el indice de ListaUndoRedo"""
		if self.__index<self.lista.tama:
			self.__index+=1

	def redo(self):
		"""Realiza un redo, desplazando el indice de ListaUndoRedo"""
		if self.__index>0:
			self.__index-=1

	def puedoundo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un undo"""
		return ((self.__index<self.lista.tama) and (len(self.lista)>1) and (self.__index<(len(self.lista)-1)))

	def puedoredo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un redo"""
		return self.__index>0

	def guardarestado(self):
		"""guarda un estado en la lista de estados, creando una copia a partir del último"""
		tmp=self.lista[self.__index]
		import copy
		objeto= copy.deepcopy(tmp)
		self.lista.encolar(objeto,self.__index)
	
	def insertarestado(self,estado):
		"""Introduce un estado y lo encola"""
		self.lista.encolar(estado,self.__index)

	def actual(self):
		"""Devuelve el estado vigente"""
		return self.lista[self.__index]

class InterfazDatos:
	"""Esta clase tiene como objetivo almacenar los datos que seran manipulados en la ejecución del programa.
	Estos datos son un conjunto de variables, y un conjunto de registros que se ajustan a dichas variables.
	La clase ofrece multiples formas de accesso a los datos internos"""

	def __init__(self,portero,gestorfiltro,gestoragrupacion):
		"""Inicialización del contenedor principal"""
		self.__portero=portero
		self.__gestorfiltro=gestorfiltro
		self.__gestoragrupacion=gestoragrupacion

	#FUNCIONES PUBLICAS

	def __getitem__(self,indice): #Acceso por []
		"""Devuelve el item segun un indice numerico, un nombre, o un objeto variable
			Si el indice es un indice, se entiende que se quiere acceder a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere acceder a la columna
		"""
		if indice.__class__==types.IntType:
			return self.__portero.actual().registros[indice]
		else:
			return self.getCol(indice,filtrado=True)
			
	def __setitem__(self,indice,valor): 
		"""
			Modificacion por []
			Si el indice es un entero, se entiende que se quiere modificar a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere modificar a la columna
		"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros[indice]=valor
		else: 
			#Añade los registros que no existen
			for registro in range(len(valor)-len(self.__portero.actual().registros)):
				self.appReg()

			for registro in range(len(self.__portero.actual().registros)):
				self.__portero.actual().registros[registro][indice]=valor[registro]

	def __delitem__(self,indice):
		"""define el borrado con la función del, borra un registro"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros.__delitem__(indice)

	def borrartodo(self):
		"""Borra todos los datos"""
		del self.__portero.actual().registros[0:self.numReg()]
		del self.__portero.actual().variables[0:self.numVar()]

	def getVar(self,indice=-1):
		"""Devuelve la variable indicada por el indice, por defecto la última"""
		return self.__portero.actual().variables[self.__portero.actual().variables.obtenerindice(indice)] 

	def setVar(self,indice,variable):
		"""Sobreescribe la variable asociada al indice.
			Hace una conversión de tipos según la representación interna de estos"""
		#Debe comprobar la correctitud del array y lanzar una excepcion si no vale
		array=[]
		for elemento in self.getCol(indice):
			array.append(repr(elemento))
		self.__portero.actual().variables[indice]=variable
		for registro in self.__portero.actual().registros:
			registro[indice]=variable.nuevoitem(array[indice])

	def numReg(self):
		"""Devuelve el número de registros almacenados"""
		return len(self.__portero.actual().registros)

	def numVar(self):
		"""Devuelve el número de variables almacenadas"""
		return len(self.__portero.actual().variables)

	def appReg(self,dato=None):
		"""Añade un dato.
		acepta tanto un objeto registro como un objeto lista """
		if isinstance(dato,Registro):
			self.__portero.actual().registros.append(dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.append(registro)

	def insReg(self,pos,dato):
		"""inserta un registro en la posicion pos"""
		if isinstance(dato,Registro):
			self.__portero.actual().registros.insert(pos,dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.insert(pos,registro)
		

	def appVar(self,nombre=None,tipo="Entero",valores={},valorpordefecto=0,descripcion="",expresion=None):  
		"""Añade una variable de tipo tipo al array de variables. 
		Rellena los registros con el valor por defecto, salvo que se indique una expresion
		"""
		if not nombre:
			nombre="VAR"+str(self.numVar())
		if tipo=="Entero":
			self.__portero.actual().variables.append(variables.TEntero(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Booleano":
			self.__portero.actual().variables.append(variables.TBooleano(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Real":
			self.__portero.actual().variables.append(variables.TReal(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Fecha":
			self.__portero.actual().variables.append(variables.TFecha(nombre,valores,valorpordefecto,descripcion)) 
		else:
			raise TypeError #PENDIENTE: Averiguar el tipo de excepción adecuado
		if expresion:
			for var in self.__portero.actual().variables:
				numero=self.__portero.actual().variables.obtenerindice(var)
				expresion=expresion.replace(var.nombre,"float(self.__getitem__(i)["+repr(numero)+"])")
			for i in range(self.numReg()):
				self.__portero.actual().registros[i].append(eval(expresion)) 
		else: #No nos pasan expresion, rellenamos con los valores por defecto
			for i in range(self.numReg()):
				self.__portero.actual().registros[i].append(valorpordefecto)

	def arrTit(self,tipo=None):
		"""
		Devuelve una lista con todos los nombres de las variables
		opcionalmente podemos filtrar segun un tipo
		"""
		if not tipo:
			return [variable.nombre for variable in self.__portero.actual().variables]
		else:
			lista=[]
			for variable in self.__portero.actual().variables:
				if variable.tipo==tipo:
					lista.append(variable.nombre)
			return lista
				
	def arrVar(self):
		"""Devuelve la lista con todas las variables"""
		return [var for var in self.__portero.actual().variables]
	
	def getCasos(self,indice):
		"""Devuelve todos los casos existentes de una determinada variable"""
		lista=[]
		for valor in self.__portero.actual().registros:
			if valor[self.__portero.actual().variables.obtenerindice(indice)] not in lista:
				lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])
		lista.sort()
		return lista

	def getCol(self,indice,filtrado=False,agrupacion=False):
		"""Devuelve un array con todos los valores asociados a la variable representada por indice 
		   La lista quedará ordenada
		"""
		#TODO: Pendiente de rediseño
		lista=[]
		if not agrupacion:
			#No hay caso
			if not filtrado or not self.__gestorfiltro.variable:
				#No Hay filtrado
				for valor in self.__portero.actual().registros:
					lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])
			else:
				#Hay filtrado
				for valor in self.__portero.actual().registros:
					if valor[self.__portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)]:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

		else:
			#Hay caso
			if not filtrado or not self.__gestorfiltro.variable:
				#No hay filtrado
				for valor in self.__portero.actual().registros:
					lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

		return lista

	def insertarestado(self,estado):
		"""Inserta un estado tal cual"""
		self.__portero.insertarestado(estado)

==> driza-0.5.3.tar.gz: driza-0.5.3/Driza/datos.py <==
#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Datos: Todas las clases relacionadas con los datos, y sus correspondientes metodos

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import variables
import types
from qt import QString

class Registro(list):
	"""Almacena un registro, permitiendo el acceso tanto por el indice, como por el nombre de variable. 
	Esta clase garantiza que la lista de valores se ajusta a los tipos de la lista de variables
	Hereda de list"""

	def __init__(self,variables):
		""" 
		Clase constructora
		variables es el array de variables al que esta asociado el registro
		Añade los valores por defecto para todos los atributos
		"""
		#VARIABLES PRIVADAS
		self.__variables=variables

		list.__init__(self)
		for x in self.__variables.arr_defaults():
			self.append(x)	
	
	#FUNCIONES PUBLICAS
	def __getitem__(self,indice):
		""" Redefinicion del [] de la lista. 
		Si nos pasan una ristra, comprobamos si se corresponde a alguna variable
		Si nos pasan un objeto, comprobamos si se corresponde a alguna variable
		Si nos pasan un entero, devolvemos el valor de esa posicion
		"""
		miindice=self.__variables.obtenerindice(indice)
		return list.__getitem__(self,miindice)

	def __setitem__(self,indice,valor):
		""" 
		Redefinicion de []= de la lista.  Almacena en el valor convertido a formato interno por la variable asociada
		"""
		miindice=self.__variables.obtenerindice(indice)
		if not isinstance(valor,str):
			valor=repr(valor)
		list.__setitem__(self,miindice,self.__variables[miindice].nuevoitem(valor))

	def append(self,valor):
		""" 
		Redefinicion del append de lista. Añade al registro el valor pasado en el formato interno de la variable asociada 
		"""
		#TODO: Este append es problemático, puesto que depende de que la variable ya exista y ademas sea la última. Pendiente de rediseñar
		intermedio=self.__variables[self.__len__()].nuevoitem(valor)
		list.append(self,intermedio)
		
	def set(self,lista):
		""" Esta funcion establece los valores del registro, teniendo como argumento una lista o derivado """
		i=0
		for valor in lista:
			self.__setitem__(i,valor)
			i+=1


class ListaVar(list): 
	"""ListaVar hereda de list. Su utilidad es almacenar un conjunto de variables, permitiendo recuperarlas no solo por el indice, sino por el nombre."""

	def __getcopy__(self):
		resultado=ListaVar()
		for i in self:
			resultado.append(i)
		return resultado

	#FUNCIONES PUBLICAS

	def arr_defaults(self):
		"""Devuelve una lista con las ristras de todos los valores por defecto"""
		return [x.valorpordefecto for x in self]

	def obtenerindice(self,parametro):
		"""
 		Permite obtener el indice de una variable a partir de su nombre ,del objeto, o de un entero.
		Devuelve un entero.
		"""
		if isinstance(parametro,QString):
			parametro=parametro.latin1()

		if parametro.__class__==types.StringType:
			for iter in self:  
				if iter.nombre == parametro:
					return self.index(iter)
			return False
		elif parametro.__class__==types.IntType:
			return parametro
		else: #El parametro lo tomamos como una referencia
			if parametro in self:
				return self.index(parametro)
			return False


class ContenedorRegVar:
	"""Esta clase almacena los datos del programa. Tiene dos miembros, una lista de registros y una lista de variables"""
	def __init__(self,registros=None,variables=None):
		"""Inicialización del contenedor principal"""
		if not registros: 
			registros=[]
		self.registros=registros #Un array para datos
		if not variables: 
			variables=ListaVar()
		self.variables=variables #Un array de variables 

	def __deepcopy__(self,memo):
		"""Reimplementación de __deepcopy__, permite una copia del objeto"""
		nuevalista=self.variables.__getcopy__()
		nuevalistaregistros=list()
		for registro in self.registros:
			nuevoregistro=Registro(nuevalista)
			for x in range(len(registro)):
				nuevoregistro[x]=nuevalista[x].nuevoitem(repr(registro[x]))
			nuevalistaregistros.append(nuevoregistro)
		resultado= ContenedorRegVar(nuevalistaregistros,nuevalista)
		memo[id(self)]=resultado
		return resultado




class ListaUndoRedo(list):
	"""ListaUndoRedo hereda de lista. Es similar a una cola lifo"""
	def __init__(self,tama):
		self.tama=tama
	def encolar(self,objeto,index=0):
		"""Si pasan un indice es que quieren encolar desde alguna posición"""
		if index>=self.tama:
			pass #error
		else:
			if self.__len__()==self.tama:
				self.__delitem__(-1)
			self.__delslice__(0,index)
			self.insert(0,objeto)

class PorteroDatos:
	"""Clase de portera. Gestiona una lista con todas las copias de los datos, determinando cual es la vigente y permitiendo el undo y el redo"""
	def __init__(self):
		self.__index=0 #Variable que lleva el indice
		self.lista=ListaUndoRedo(5)
		self.lista.encolar(ContenedorRegVar())

	def undo(self):
		"""Realiza un undo, desplazando el indice de ListaUndoRedo"""
		if self.__index<self.lista.tama:
			self.__index+=1

	def redo(self):
		"""Realiza un redo, desplazando el indice de ListaUndoRedo"""
		if self.__index>0:
			self.__index-=1

	def puedoundo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un undo"""
		return ((self.__index<self.lista.tama) and (len(self.lista)>1) and (self.__index<(len(self.lista)-1)))

	def puedoredo(self):
		"""Devuelve un valor lógico indicando si es posible realizar un redo"""
		return self.__index>0

	def guardarestado(self):
		"""guarda un estado en la lista de estados, creando una copia a partir del último"""
		tmp=self.lista[self.__index]
		import copy
		objeto= copy.deepcopy(tmp)
		self.lista.encolar(objeto,self.__index)
	
	def insertarestado(self,estado):
		"""Introduce un estado y lo encola"""
		self.lista.encolar(estado,self.__index)

	def actual(self):
		"""Devuelve el estado vigente"""
		return self.lista[self.__index]

class InterfazDatos:
	"""Esta clase tiene como objetivo almacenar los datos que seran manipulados en la ejecución del programa.
	Estos datos son un conjunto de variables, y un conjunto de registros que se ajustan a dichas variables.
	La clase ofrece multiples formas de accesso a los datos internos"""

	def __init__(self,portero,gestorfiltro,gestoragrupacion):
		"""Inicialización del contenedor principal"""
		self.__portero=portero
		self.__gestorfiltro=gestorfiltro
		self.__gestoragrupacion=gestoragrupacion

	#FUNCIONES PUBLICAS

	def __getitem__(self,indice): #Acceso por []
		"""Devuelve el item segun un indice numerico, un nombre, o un objeto variable
			Si el indice es un indice, se entiende que se quiere acceder a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere acceder a la columna
		"""
		if indice.__class__==types.IntType:
			return self.__portero.actual().registros[indice]
		else:
			return self.getCol(indice,filtrado=True)
			
	def __setitem__(self,indice,valor): 
		"""
			Modificacion por []
			Si el indice es un entero, se entiende que se quiere modificar a un registro
			Si el indice es un nombre o el objeto variable, se entiende que se quiere modificar a la columna
		"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros[indice]=valor
		else: 
			#Añade los registros que no existen
			for registro in range(len(valor)-len(self.__portero.actual().registros)):
				self.appReg()

			for registro in range(len(self.__portero.actual().registros)):
				self.__portero.actual().registros[registro][indice]=valor[registro]

	def __delitem__(self,indice):
		"""define el borrado con la función del, borra un registro"""
		if indice.__class__==types.IntType:
			self.__portero.actual().registros.__delitem__(indice)

	def borrartodo(self):
		"""Borra todos los datos"""
		del self.__portero.actual().registros[0:self.numReg()]
		del self.__portero.actual().variables[0:self.numVar()]

	def getVar(self,indice=-1):
		"""Devuelve la variable indicada por el indice, por defecto la última"""
		return self.__portero.actual().variables[self.__portero.actual().variables.obtenerindice(indice)] 

	def setVar(self,indice,variable,columna=None):
		"""Sobreescribe la variable asociada al indice.
			Hace una conversión de tipos según la representación interna de estos
			Si se le pasa la columna, entonces sobreescribira todos los registros asociados a la variable
			"""
		#Debe comprobar la correctitud del array y lanzar una excepcion si no vale
		array=[]
		if not columna:
			#Conversion directa (Inestable)
			for elemento in self.getCol(indice):
				array.append(repr(elemento))
			self.__portero.actual().variables[indice]=variable
			for registro in self.__portero.actual().registros:
				registro[indice]=variable.nuevoitem(array[indice])

		else:
			#Nos pasan la columna con los datos
			self.__portero.actual().variables[indice]=variable
			for registro in self.__portero.actual().registros:
				registro[indice]=variable.nuevoitem(columna[indice])
		#TODO: Continuar con el caso de la columna

	def numReg(self):
		"""Devuelve el número de registros almacenados"""
		return len(self.__portero.actual().registros)

	def numVar(self):
		"""Devuelve el número de variables almacenadas"""
		return len(self.__portero.actual().variables)

	def appReg(self,dato=None):
		"""Añade un dato.
		acepta tanto un objeto registro como un objeto lista """
		if isinstance(dato,Registro):
			self.__portero.actual().registros.append(dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.append(registro)

	def insReg(self,pos,dato):
		"""inserta un registro en la posicion pos"""
		if isinstance(dato,Registro):
			self.__portero.actual().registros.insert(pos,dato)
		else:
			registro=Registro(self.__portero.actual().variables)
			if dato.__class__==types.ListType: #Controlar otros casos
				registro.set(dato)
			self.__portero.actual().registros.insert(pos,registro)
		

	def appVar(self,nombre=None,tipo="Entero",valores={},valorpordefecto=0,descripcion="",expresion=None):  
		"""Añade una variable de tipo tipo al array de variables. 
		Rellena los registros con el valor por defecto, salvo que se indique una expresion
		"""
		if not nombre:
			nombre="VAR"+str(self.numVar())
		if tipo=="Entero":
			self.__portero.actual().variables.append(variables.TEntero(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Booleano":
			self.__portero.actual().variables.append(variables.TBooleano(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Real":
			self.__portero.actual().variables.append(variables.TReal(nombre,valores,valorpordefecto,descripcion)) 
		elif tipo=="Fecha":
			self.__portero.actual().variables.append(variables.TFecha(nombre,valores,valorpordefecto,descripcion)) 
		else:
			raise TypeError #PENDIENTE: Averiguar el tipo de excepción adecuado
		if expresion:
			for var in self.__portero.actual().variables:
				numero=self.__portero.actual().variables.obtenerindice(var)
				expresion=expresion.replace(var.nombre,"float(self.__getitem__(i)["+repr(numero)+"])")
			for i in range(self.numReg()):
				self.__portero.actual().registros[i].append(eval(expresion)) 
		else: #No nos pasan expresion, rellenamos con los valores por defecto
			for i in range(self.numReg()):
				self.__portero.actual().registros[i].append(valorpordefecto)

	def arrTit(self,tipo=None):
		"""
		Devuelve una lista con todos los nombres de las variables
		opcionalmente podemos filtrar segun un tipo
		"""
		if not tipo:
			return [variable.nombre for variable in self.__portero.actual().variables]
		else:
			lista=[]
			for variable in self.__portero.actual().variables:
				if variable.tipo==tipo:
					lista.append(variable.nombre)
			return lista
				
	def arrVar(self):
		"""Devuelve la lista con todas las variables"""
		return [var for var in self.__portero.actual().variables]
	
	def getCasos(self,indice=None):
		"""Devuelve todos los casos existentes de una determinada variable"""
		lista=[]
		if not indice:
			indice=self.__gestoragrupacion.variable
		indiceentero=self.__portero.actual().variables.obtenerindice(indice)
		for valor in self.__portero.actual().registros:
			if valor[indiceentero] not in lista:
				lista.append(valor[indiceentero])
		lista.sort()
		return lista

	def getCol(self,indice,filtrado=True,caso=False):
		"""Devuelve un array con todos los valores asociados a la variable representada por indice 
		   La lista quedará ordenada
		"""
		lista=[]
		if not caso:
			#No hay caso
			if not filtrado or not self.__gestorfiltro.variable:
				#No Hay filtrado
				for valor in self.__portero.actual().registros:
					lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])
			else:
				#Hay filtrado
				for valor in self.__portero.actual().registros:
					if valor[self.__portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)]:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])

		else:
			#Hay caso
		#	assert self.__portero.actual().obtenerindice(indice)!=self.__portero.actual().obtenerindice(self.__gestoragrupacion.variable)
			#El caso debe pertenecer a una variable diferente a la que representa el indice
			if not filtrado or not self.__gestorfiltro.variable:
				#No hay filtrado
				for valor in self.__portero.actual().registros:
					if caso==valor[self.__portero.actual().variables.obtenerindice(self.__gestoragrupacion.variable)]:
						lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])
			else:
				#Hay filtrado
				for valor in self.__portero.actual().registros:
					if valor[self.__portero.actual().variables.obtenerindice(self.__gestorfiltro.variable)]:
						if caso==valor[self.__portero.actual().variables.obtenerindice(self.__gestoragrupacion.variable)]:
							lista.append(valor[self.__portero.actual().variables.obtenerindice(indice)])


		return lista

	def insertarestado(self,estado):
		"""Inserta un estado tal cual"""
		self.__portero.insertarestado(estado)

	def agrupacion(self):
		"""Determina si existe una variable de agrupacion o no"""
		if self.__gestoragrupacion.variable:
			return True
		else:
			return False
